@@ -1,24 +1,16 @@
-/*************************************************************************
- *  Copyright (C) 2008, 2010 by Volker Lanz <vl@fidra.de>                *
- *  Copyright (C) 2016 by Teo Mrnjavac <teo@kde.org>                     *
- *                                                                       *
- *  This program is free software; you can redistribute it and/or        *
- *  modify it under the terms of the GNU General Public License as       *
- *  published by the Free Software Foundation; either version 3 of       *
- *  the License, or (at your option) any later version.                  *
- *                                                                       *
- *  This program is distributed in the hope that it will be useful,      *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *  GNU General Public License for more details.                         *
- *                                                                       *
- *  You should have received a copy of the GNU General Public License    *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
- *************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2008-2012 Volker Lanz <vl@fidra.de>
+    SPDX-FileCopyrightText: 2008 Laurent Montel <montel@kde.org>
+    SPDX-FileCopyrightText: 2015 Chris Campbell <c.j.campbell@ed.ac.uk>
+    SPDX-FileCopyrightText: 2016-2019 Andrius Štikonas <andrius@stikonas.eu>
+    SPDX-FileCopyrightText: 2015-2016 Teo Mrnjavac <teo@kde.org>
+    SPDX-FileCopyrightText: 2016 Chantara Tith <tith.chantara@gmail.com>
 
-#if !defined(PARTITIONTABLE__H)
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
-#define PARTITIONTABLE__H
+#ifndef KPMCORE_PARTITIONTABLE_H
+#define KPMCORE_PARTITIONTABLE_H
 
 #include "util/libpartitionmanagerexport.h"
 
@@ -44,13 +36,13 @@ class QTextStream;
 */
 class LIBKPMCORE_EXPORT PartitionTable : public PartitionNode
 {
-    Q_DISABLE_COPY(PartitionTable)
+    PartitionTable &operator=(const PartitionTable &) = delete;
 
     friend class CoreBackend;
     friend LIBKPMCORE_EXPORT QTextStream& operator<<(QTextStream& stream, const PartitionTable& ptable);
 
 public:
-    enum TableType : qint8 {
+    enum TableType : int8_t {
         unknownTableType = -1,
 
         aix,
@@ -64,37 +56,58 @@ public:
         mac,
         pc98,
         amiga,
-        sun
+        sun,
+        vmd,  /* Volume Manager Device */
+        none, /* Single FileSystem devices */
     };
 
     /** Partition flags */
-    enum Flag : qint32 {
-        FlagNone = 0,
-        FlagBoot = 1,
-        FlagRoot = 2,
-        FlagSwap = 4,
-        FlagHidden = 8,
-        FlagRaid = 16,
-        FlagLvm = 32,
-        FlagLba = 64,
-        FlagHpService = 128,
-        FlagPalo = 256,
-        FlagPrep = 512,
-        FlagMsftReserved = 1024,
-        FlagBiosGrub = 2048,
-        FlagAppleTvRecovery = 4096,
-        FlagDiag = 8192,
-        FlagLegacyBoot = 16384,
-        FlagMsftData = 32768,
-        FlagIrst = 65536,
-        FlagEsp = 131072
+    enum Flag : uint32_t {
+        None = 0x0,
+        Boot = 0x1,
+        Root = 0x2,
+        Swap = 0x4,
+        Hidden = 0x8,
+        Raid = 0x10,
+        Lvm = 0x20,
+        Lba = 0x40,
+        HpService = 0x80,
+        Palo = 0x100,
+        Prep = 0x200,
+        MsftReserved = 0x400,
+        BiosGrub = 0x800,
+        AppleTvRecovery = 0x1000,
+        Diag = 0x2000,
+        LegacyBoot = 0x4000,
+        MsftData = 0x8000,
+        Irst = 0x100000,
+        FlagNone [[deprecated("Use PartitionTable::Flag::None")]] = None,
+        FlagBoot [[deprecated("Use PartitionTable::Flag::Boot")]] = Boot,
+        FlagRoot [[deprecated("Use PartitionTable::Flag::Root")]] = Root,
+        FlagSwap [[deprecated("Use PartitionTable::Flag::Swap")]] = Swap,
+        FlagHidden [[deprecated("Use PartitionTable::Flag::Hidden")]] = Hidden,
+        FlagRaid [[deprecated("Use PartitionTable::Flag::Raid")]] = Raid,
+        FlagLvm [[deprecated("Use PartitionTable::Flag::Lvm")]] = Lvm,
+        FlagLba [[deprecated("Use PartitionTable::Flag::Lba")]] = Lba,
+        FlagHpService [[deprecated("Use PartitionTable::Flag::HpService")]] = HpService,
+        FlagPalo [[deprecated("Use PartitionTable::Flag::Palo")]] = Palo,
+        FlagPrep [[deprecated("Use PartitionTable::Flag::Prep")]] = Prep,
+        FlagMsftReserved [[deprecated("Use PartitionTable::Flag::MsftReserved")]] = MsftReserved,
+        FlagBiosGrub [[deprecated("Use PartitionTable::Flag::BiosGrub")]] = BiosGrub,
+        FlagAppleTvRecovery [[deprecated("Use PartitionTable::Flag::AppleTvRecovery")]] = AppleTvRecovery,
+        FlagDiag [[deprecated("Use PartitionTable::Flag::Diag")]] = Diag,
+        FlagLegacyBoot [[deprecated("Use PartitionTable::Flag::LegacyBoot")]] = LegacyBoot,
+        FlagMsftData [[deprecated("Use PartitionTable::Flag::MsftData")]] = MsftData,
+        FlagIrst [[deprecated("Use PartitionTable::Flag::Irst")]] = Irst,
+        FlagEsp [[deprecated("Use PartitionTable::Flag::Boot")]] = Boot
     };
 
     Q_DECLARE_FLAGS(Flags, Flag)
 
 public:
-    PartitionTable(TableType type, qint64 first_usable, qint64 last_usable);
-    ~PartitionTable();
+    PartitionTable(TableType type, qint64 firstUsable, qint64 lastUsable);
+    PartitionTable(const PartitionTable& other);
+    ~PartitionTable() override;
 
 public:
     PartitionNode* parent() override {
@@ -124,14 +137,15 @@ public:
 
     qint64 freeSectorsBefore(const Partition& p) const;
     qint64 freeSectorsAfter(const Partition& p) const;
+    qint64 freeSectors() const;
 
     bool hasExtended() const;
     Partition* extended() const;
 
     PartitionRole::Roles childRoles(const Partition& p) const;
 
-    int numPrimaries() const;
-    int maxPrimaries() const {
+    qint32 numPrimaries() const;
+    qint32 maxPrimaries() const {
         return m_MaxPrimaries;    /**< @return max number of primary partitions this PartitionTable can handle */
     }
 
@@ -149,14 +163,22 @@ public:
         return m_LastUsable;
     }
 
+    void setFirstUsableSector(qint64 s) {
+        m_FirstUsable = s;
+    }
+    void setLastUsableSector(qint64 s) {
+        m_LastUsable = s;
+    }
+
     void updateUnallocated(const Device& d);
-    void insertUnallocated(const Device& d, PartitionNode* p, qint64 start) const;
+    void insertUnallocated(const Device& d, PartitionNode* p, qint64 start);
 
     bool isSectorBased(const Device& d) const;
 
-    static QList<Flag> flagList();
+    static const QList<Flag> flagList();
     static QString flagName(Flag f);
     static QStringList flagNames(Flags f);
+    static PartitionTable::Flags flagsFromList(const QStringList list);
 
     static bool getUnallocatedRange(const Device& device, PartitionNode& parent, qint64& start, qint64& end);
 
@@ -168,7 +190,7 @@ public:
 
     static PartitionTable::TableType nameToTableType(const QString& n);
     static QString tableTypeToName(TableType l);
-    static qint64 maxPrimariesForTableType(TableType l);
+    static qint32 maxPrimariesForTableType(TableType l);
     static bool tableTypeSupportsExtended(TableType l);
     static bool tableTypeIsReadOnly(TableType l);
 
@@ -176,12 +198,6 @@ protected:
     void setMaxPrimaries(qint32 n) {
         m_MaxPrimaries = n;
     }
-    void setFirstUsableSector(qint64 s) {
-        m_FirstUsable = s;
-    }
-    void setLastUsableSector(qint64 s) {
-        m_LastUsable = s;
-    }
 
 private:
     Partitions m_Children;
